@@ -4,10 +4,9 @@ import intercom
 import json
 import unittest
 
-from intercom import Intercom
-from intercom import Request
+from intercom.client import Client
+from intercom.request import Request
 from intercom import UnexpectedError
-from mock import Mock
 from mock import patch
 from nose.tools import assert_raises
 from nose.tools import eq_
@@ -18,53 +17,62 @@ from tests.unit import mock_response
 
 class RequestTest(unittest.TestCase):
 
+    def setUp(self):
+        self.client = Client()
+
     @istest
     def it_raises_resource_not_found(self):
-        resp = mock_response('{}', status_code=404)
+        resp = mock_response(None, status_code=404)
         with patch('requests.request') as mock_method:
             mock_method.return_value = resp
             with assert_raises(intercom.ResourceNotFound):
-                Request.send_request_to_path('GET', 'notes', ('x', 'y'), resp)
+                request = Request('GET', 'notes')
+                request.send_request_to_path('', ('x', 'y'), resp)
 
     @istest
     def it_raises_authentication_error_unauthorized(self):
-        resp = mock_response('{}', status_code=401)
+        resp = mock_response(None, status_code=401)
         with patch('requests.request') as mock_method:
             mock_method.return_value = resp
             with assert_raises(intercom.AuthenticationError):
-                Request.send_request_to_path('GET', 'notes', ('x', 'y'), resp)
+                request = Request('GET', 'notes')
+                request.send_request_to_path('', ('x', 'y'), resp)
 
     @istest
     def it_raises_authentication_error_forbidden(self):
-        resp = mock_response('{}', status_code=403)
+        resp = mock_response(None, status_code=403)
         with patch('requests.request') as mock_method:
             mock_method.return_value = resp
             with assert_raises(intercom.AuthenticationError):
-                Request.send_request_to_path('GET', 'notes', ('x', 'y'), resp)
+                request = Request('GET', 'notes')
+                request.send_request_to_path('', ('x', 'y'), resp)
 
     @istest
     def it_raises_server_error(self):
-        resp = Mock(encoding="utf-8", content='{}', status_code=500)
+        resp = mock_response(None, status_code=500)
         with patch('requests.request') as mock_method:
             mock_method.return_value = resp
             with assert_raises(intercom.ServerError):
-                Request.send_request_to_path('GET', 'notes', ('x', 'y'), resp)
+                request = Request('GET', 'notes')
+                request.send_request_to_path('', ('x', 'y'), resp)
 
     @istest
     def it_raises_bad_gateway_error(self):
-        resp = mock_response('{}', status_code=502)
+        resp = mock_response(None, status_code=502)
         with patch('requests.request') as mock_method:
             mock_method.return_value = resp
             with assert_raises(intercom.BadGatewayError):
-                Request.send_request_to_path('GET', 'notes', ('x', 'y'), resp)
+                request = Request('GET', 'notes')
+                request.send_request_to_path('', ('x', 'y'), resp)
 
     @istest
     def it_raises_service_unavailable_error(self):
-        resp = mock_response('{}', status_code=503)
+        resp = mock_response(None, status_code=503)
         with patch('requests.request') as mock_method:
             mock_method.return_value = resp
             with assert_raises(intercom.ServiceUnavailableError):
-                Request.send_request_to_path('GET', 'notes', ('x', 'y'), resp)
+                request = Request('GET', 'notes')
+                request.send_request_to_path('', ('x', 'y'), resp)
 
     @istest
     def it_raises_an_unexpected_typed_error(self):
@@ -79,10 +87,10 @@ class RequestTest(unittest.TestCase):
         }
         content = json.dumps(payload).encode('utf-8')
         resp = mock_response(content)
-        with patch('requests.request') as mock_method:
+        with patch('requests.sessions.Session.request') as mock_method:
             mock_method.return_value = resp
             try:
-                Intercom.get('/users')
+                self.client.get('/users', {})
                 self.fail('UnexpectedError not raised.')
             except (UnexpectedError) as err:
                 ok_("The error of type 'hopper' is not recognized" in err.message)  # noqa
@@ -101,10 +109,10 @@ class RequestTest(unittest.TestCase):
         }
         content = json.dumps(payload).encode('utf-8')
         resp = mock_response(content)
-        with patch('requests.request') as mock_method:
+        with patch('requests.sessions.Session.request') as mock_method:
             mock_method.return_value = resp
             try:
-                Intercom.get('/users')
+                self.client.get('/users', {})
                 self.fail('UnexpectedError not raised.')
             except (UnexpectedError) as err:
                 ok_("An unexpected error occured." in err.message)
@@ -127,10 +135,10 @@ class RequestTest(unittest.TestCase):
 
             content = json.dumps(payload).encode('utf-8')
             resp = mock_response(content)
-            with patch('requests.request') as mock_method:
+        with patch('requests.sessions.Session.request') as mock_method:
                 mock_method.return_value = resp
                 with assert_raises(intercom.BadRequestError):
-                    Intercom.get('/users')
+                    self.client.get('/users', {})
 
     @istest
     def it_raises_an_authentication_error(self):
@@ -148,10 +156,10 @@ class RequestTest(unittest.TestCase):
 
             content = json.dumps(payload).encode('utf-8')
             resp = mock_response(content)
-            with patch('requests.request') as mock_method:
+            with patch('requests.sessions.Session.request') as mock_method:
                 mock_method.return_value = resp
                 with assert_raises(intercom.AuthenticationError):
-                    Intercom.get('/users')
+                    self.client.get('/users', {})
 
     @istest
     def it_raises_resource_not_found_by_type(self):
@@ -166,10 +174,10 @@ class RequestTest(unittest.TestCase):
         }
         content = json.dumps(payload).encode('utf-8')
         resp = mock_response(content)
-        with patch('requests.request') as mock_method:
+        with patch('requests.sessions.Session.request') as mock_method:
             mock_method.return_value = resp
             with assert_raises(intercom.ResourceNotFound):
-                Intercom.get('/users')
+                self.client.get('/users', {})
 
     @istest
     def it_raises_rate_limit_exceeded(self):
@@ -184,10 +192,10 @@ class RequestTest(unittest.TestCase):
         }
         content = json.dumps(payload).encode('utf-8')
         resp = mock_response(content)
-        with patch('requests.request') as mock_method:
+        with patch('requests.sessions.Session.request') as mock_method:
             mock_method.return_value = resp
             with assert_raises(intercom.RateLimitExceeded):
-                Intercom.get('/users')
+                self.client.get('/users', {})
 
     @istest
     def it_raises_a_service_unavailable_error(self):
@@ -202,10 +210,10 @@ class RequestTest(unittest.TestCase):
         }
         content = json.dumps(payload).encode('utf-8')
         resp = mock_response(content)
-        with patch('requests.request') as mock_method:
+        with patch('requests.sessions.Session.request') as mock_method:
             mock_method.return_value = resp
             with assert_raises(intercom.ServiceUnavailableError):
-                Intercom.get('/users')
+                self.client.get('/users', {})
 
     @istest
     def it_raises_a_multiple_matching_users_error(self):
@@ -220,10 +228,28 @@ class RequestTest(unittest.TestCase):
         }
         content = json.dumps(payload).encode('utf-8')
         resp = mock_response(content)
-        with patch('requests.request') as mock_method:
+        with patch('requests.sessions.Session.request') as mock_method:
             mock_method.return_value = resp
             with assert_raises(intercom.MultipleMatchingUsersError):
-                Intercom.get('/users')
+                self.client.get('/users', {})
+
+    @istest
+    def it_raises_token_unauthorized(self):
+        payload = {
+            'type': 'error.list',
+            'errors': [
+                {
+                    'type': 'token_unauthorized',
+                    'message': 'The PAT is not authorized for this action.'
+                }
+            ]
+        }
+        content = json.dumps(payload).encode('utf-8')
+        resp = mock_response(content)
+        with patch('requests.sessions.Session.request') as mock_method:
+            mock_method.return_value = resp
+            with assert_raises(intercom.TokenUnauthorizedError):
+                self.client.get('/users', {})
 
     @istest
     def it_handles_no_error_type(self):
@@ -239,10 +265,10 @@ class RequestTest(unittest.TestCase):
         }
         content = json.dumps(payload).encode('utf-8')
         resp = mock_response(content)
-        with patch('requests.request') as mock_method:
+        with patch('requests.sessions.Session.request') as mock_method:
             mock_method.return_value = resp
             with assert_raises(intercom.MultipleMatchingUsersError):
-                Intercom.get('/users')
+                self.client.get('/users', {})
 
         payload = {
             'errors': [
@@ -256,22 +282,24 @@ class RequestTest(unittest.TestCase):
         }
         content = json.dumps(payload).encode('utf-8')
         resp = mock_response(content)
-        with patch('requests.request') as mock_method:
+        with patch('requests.sessions.Session.request') as mock_method:
             mock_method.return_value = resp
             with assert_raises(intercom.BadRequestError):
-                Intercom.get('/users')
+                self.client.get('/users', {})
 
     @istest
     def it_handles_empty_responses(self):
         resp = mock_response('', status_code=202)
         with patch('requests.request') as mock_method:
             mock_method.return_value = resp
-            Request.send_request_to_path('GET', 'events', ('x', 'y'), resp)
+            request = Request('GET', 'events')
+            request.send_request_to_path('', ('x', 'y'), resp)
 
         resp = mock_response(' ', status_code=202)
         with patch('requests.request') as mock_method:
             mock_method.return_value = resp
-            Request.send_request_to_path('GET', 'events', ('x', 'y'), resp)
+            request = Request('GET', 'events')
+            request.send_request_to_path('', ('x', 'y'), resp)
 
     @istest
     def it_handles_no_encoding(self):
@@ -281,7 +309,8 @@ class RequestTest(unittest.TestCase):
 
         with patch('requests.request') as mock_method:
             mock_method.return_value = resp
-            Request.send_request_to_path('GET', 'events', ('x', 'y'), resp)
+            request = Request('GET', 'events')
+            request.send_request_to_path('', ('x', 'y'), resp)
 
     @istest
     def it_needs_encoding_or_apparent_encoding(self):
@@ -297,10 +326,12 @@ class RequestTest(unittest.TestCase):
         with patch('requests.request') as mock_method:
             mock_method.return_value = resp
             with assert_raises(TypeError):
-                Request.send_request_to_path('GET', 'events', ('x', 'y'), resp)
+                request = Request('GET', 'events')
+                request.send_request_to_path('', ('x', 'y'), resp)
 
     @istest
     def it_allows_the_timeout_to_be_changed(self):
-        eq_(10, intercom.Request.timeout)
-        intercom.Request.timeout = 3
-        eq_(3, intercom.Request.timeout)
+        from intercom.request import Request
+        eq_(10, Request.timeout)
+        Request.timeout = 3
+        eq_(3, Request.timeout)
